﻿using Kivii.Domain.Lac.Lims.Extensions;
using Kivii;
using Kivii.Lims.Entities;
using Kivii.Lac.Lims.Extensions;
using Kivii.Linq;
using Kivii.Products.Entities;
using Kivii.Standards.Entities;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using Report = Kivii.Lims.Entities.Report;
using Sample = Kivii.Lims.Entities.Sample;
using Kivii.Office.OpenXml.Excel;

namespace Kivii.Domain.Lac.Lims.Transforms
{
    #region 获取报告样品等级+功能等级
    //代码初审 张梦圆 20210907
    public class Condition
    {
        public string Name { get; set; }
    }
    public enum SampleGradeType
    {
        Unsupported = 0,
        //样品等级
        SampleGrade = 100,
        //功能等级
        SampleGradeFunction = 200
    }
    [Api(Description = "获取报告样品等级")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class GetSampleGrade : RestfulExecution<Report>
    {
        public Guid ReportKvid { get; set; }
        public SampleGradeType SampleGradeType { get; set; } = SampleGradeType.SampleGrade;

        public override object OnExecution(IRequest req, IResponse res)
        {
            var rtns = new RestfulQueryResponse<Condition>();
            rtns.Results = new List<Condition>();
            #region 数据验证
            if (ReportKvid == Guid.Empty) return rtns;
            if(SampleGradeType== SampleGradeType.Unsupported) return rtns;
            #endregion
            //获取报告检验项目Kvid集合
            var connLims = KiviiContext.GetOpenedDbConnection<Report>();
            var queryDetectionsKvids = connLims.From<ReportItem>();
            queryDetectionsKvids.Where(o => o.ReportKvid == ReportKvid);
            queryDetectionsKvids.SelectDistinct(o => o.DetectionKvid);
            var detectionsKvids = connLims.SqlColumn<Guid>(queryDetectionsKvids);
            //获取对应检验值集合
            var connStandards = KiviiContext.GetOpenedDbConnection<Standard>();
            var queryDetectionValues = connStandards.From<DetectionValue>();
            queryDetectionValues.Where(o => Sql.In(o.DetectionKvid, detectionsKvids) && o.ParentKvid == Guid.Empty).OrderBy(o => o.SortId);
            var detectionValues = connStandards.Select(queryDetectionValues);
            var conditions = detectionValues.Select(o => o.Condition).Distinct().ToList();
            foreach (var items in conditions)
            {
                if (items == "A类" || items == "B类" || items == "C类") continue;
                var condition = new Condition();
                if (SampleGradeType == SampleGradeType.SampleGrade)
                {
                    if (items != "优等品" && items != "一等品" && items != "合格品" && items != "二等品") continue;
                    condition.Name = items;
                }
                else {
                    if (items == "优等品" || items == "一等品" || items == "合格品" || items == "二等品") continue;
                    condition.Name = items;
                }
                rtns.Results.Add(condition);
            }
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion 

    #region 更改报告合同
    /// <summary>
    /// 更改报告协议的同时会更新检验项目的折后费
    /// </summary>
    //代码初审 张梦圆 20210907
    [Api(Description = "更改报告协议")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class ReportUpdateQuotation : RestfulExecution<Report>
    {
        public Guid ReportKvid { get; set; }

        public Guid QuotationKvid { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            if (ReportKvid == Guid.Empty) throw new Exception("请传入ReportKvid!");
            if (QuotationKvid == Guid.Empty) throw new Exception("请传入QuotationKvid!");
            #endregion
            var rtns = new RestfulQueryResponse<Report>();
            rtns.Results = new List<Report>();

            //获取报价单信息
            var connProducts = KiviiContext.GetOpenedDbConnection<Quotation>();
            var quotation = connProducts.SingleById<Quotation>(QuotationKvid);
            quotation.ThrowIfNull("未找到对应报价单!");

            //获取报价单明细
            var queryQuotationDetails = connProducts.From<QuotationDetail>();
            queryQuotationDetails.Where(o => o.QuotationKvid == QuotationKvid);
            var quotationDetails = connProducts.Select(queryQuotationDetails);

            //定义变量获取更新后的折后费
            decimal amountTest = 0;

            var connLims = KiviiContext.GetOpenedDbConnection<ReportItem>();
            //获取Report
            var ownerReport = connLims.SingleById<Report>(ReportKvid);
            //获取ReportQuotation
            var reportQuotation = connLims.SingleById<ReportQuotation>(ReportKvid);
            reportQuotation.ThrowIfNull("未找到对应报告!");
            reportQuotation.RemoveAllOnlyProperties();

            //获取ReportItems
            var queryReportItems = connLims.From<ReportItem>();
            queryReportItems.Where(o => o.ReportKvid == ReportKvid);
            var reportItems = connLims.Select(queryReportItems);

            //获取Prices集合
            //20230601此处隐藏bug，当检验项目非默认的价格类型时，只根据GoodsKvid无法获取到对应的报价单明细，需要加上PriceType参数一起查
            var goodsKvids = reportItems.Select(o => o.GoodsKvid).Distinct().ToList();
            var queryPrices = connProducts.From<Price>();
            queryPrices.Where(o => Sql.In(o.GoodsKvid, goodsKvids));
            var prices = connProducts.Select(queryPrices);
            IDbTransaction trans = null;//define a null tag
            trans = connLims.OpenTransaction();
            try
            {
                foreach (var reportItem in reportItems)
                {
                    var originalAmountPlan = reportItem.AmountPlan;
                    //首先排除目录项和排除没有关联服务的项
                    if ((reportItem.Type != ReportItemType.TestFolder && reportItem.Type != ReportItemType.Test) || reportItem.GoodsKvid == Guid.Empty) continue;
                    if (reportItem.Type == ReportItemType.Test && reportItem.ParentKvid != Guid.Empty)
                    {
                        var reportItemParent = reportItems.Where(o => o.Kvid == reportItem.ParentKvid).FirstOrDefault();
                        reportItemParent.ThrowIfNull("未找到对应检验项目!");
                        if (reportItemParent.Type == ReportItemType.TestFolder) continue;
                    }
                    //获取当前项目使用的价格
                    var currentPrice = new Price();
                    if (reportItem.PackageUnit.IsNullOrEmpty()) continue;
                    var reportItemPrices = prices.Where(o => o.GoodsKvid == reportItem.GoodsKvid&&o.PackageUnit==reportItem.PackageUnit.Split(',').FirstOrDefault()).ToList();
                    if (reportItemPrices.Count == 0) continue;
                    if (reportItemPrices.Count == 1)
                    {
                        currentPrice = reportItemPrices[0];
                    }
                    else
                    {
                        currentPrice = reportItemPrices.FirstOrDefault(o => o.IsDefault);
                        if (currentPrice == null) currentPrice = reportItemPrices[0];
                    }
                    //更新合同时，检验费恢复原价
                    reportItem.AmountPlan = currentPrice.Amount;
                    reportItem.AddOnlyProperties(o => o.AmountPlan);
                    QuotationDetail quotationDetail = quotationDetails == null ? null : quotationDetails.Where(o => o.PriceKvid == currentPrice.Kvid).FirstOrDefault();

                    //如果没有客户报价详细，直接用折扣
                    if (quotationDetail == null)
                    {
                        reportItem.Amount = (decimal)(reportItem.AmountPlan * quotation.DiscountRate);
                    }
                    else
                    {
                        reportItem.Amount = quotationDetail.Amount;
                    }
                    amountTest += reportItem.Amount;
                    reportItem.AddOnlyProperties(o => o.Amount);
                    connLims.UpdateOnly(reportItem);
                    if(originalAmountPlan!= reportItem.AmountPlan)
                    {
                        reportItem.SetAmountLog(ownerReport, "ReportUpdateQuotation", originalAmountPlan, reportItem.AmountPlan, connLims);
                    }
                }
                reportQuotation.QuotationKvid = quotation.Kvid;
                reportQuotation.QuotationName = quotation.Name;
                reportQuotation.QuotationAmount = quotation.Amount;
                reportQuotation.QuotationDiscountRate = (decimal)quotation.DiscountRate;

                var report = new Report();
                report.PopulateWith(reportQuotation);
                report.CalculateReportAmount(connLims);
                reportQuotation.AmountTest = report.AmountTest;
                reportQuotation.AmountTestPlan = report.AmountTestPlan;
                reportQuotation.QuotationAmount = report.QuotationAmount;
                reportQuotation.AddOnlyProperties(o => new { o.QuotationKvid, o.QuotationName, o.QuotationAmount, o.QuotationDiscountRate, o.AmountTest, o.AmountTestPlan });
                connLims.UpdateOnly(reportQuotation);
                reportQuotation.RemoveAllOnlyProperties();
                trans?.Commit();

                rtns.Results.Add(report);
                rtns.Total = rtns.Results.Count;
                return rtns;
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
        }
    }
    #endregion

    #region 生成报告结论
    //代码初审 张梦圆 20210907
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class ReportConclusion : RestfulExecution<Report>
    {
        public string ReportKvid { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportKvid.ThrowIfNullOrEmpty("参数异常！");
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var report = conn.SingleById<Report>(ReportKvid);
            report.ThrowIfNull("未找到指定报告！");
            if (report.StatusType != ReportStatusType.ReportPreparation) throw new Exception("报告状态非编制状态，无法生成报告结论！");
            //若报告要求为数据--直接返回。
            if (!report.TestNeedJudge || report.TestJudgement == "/")
            {
                report.Conclusion = "见检测结果页。";
                report.AddOnlyProperties(o => o.Conclusion);
            }
            else
            {
                var first = string.Empty;
                var second = string.Empty;
                var third = string.Empty;
                if (report.TypeName == "委托送样")
                {
                    first = "样品经检验";
                    report.GenerateConclusion(first, second, third);
                }
                else if (report.TypeName == "委托抽样")
                {
                    first = "样本经检验";
                    third = $"检验结论为{report.TestJudgement}";
                    report.GenerateConclusion(first, second, third);
                }
                else if (report.TypeName.Contains("监督"))
                {
                    first = "经抽样检验";
                    second = "项目";
                    third = $"检验结论为{report.TestJudgement}";
                    report.GenerateConclusion(first, second, third);
                }
                else
                {
                    throw new Exception($"暂无检验类型为{report.TypeName}的模板，无法自动生成检验结论！");
                }
            }
            report.RemoveAllOnlyProperties();
            return report;
        }
    }
    #endregion

    #region 更改报告的协议价
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class ReportUpdateQuotationAmount : RestfulExecution<ReportQuotation>
    {
        public Guid ReportKvid { get; set; }
        public decimal QuotationAmount { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportKvid.ThrowIfEmpty("参数ReportKvid异常！");
            ReportKvid.ThrowIfNull("参数ReportKvid异常！");
            QuotationAmount.ThrowIfNull("参数QuotationAmount异常!");

            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var reportQuotation = conn.SingleById<ReportQuotation>(ReportKvid);
            reportQuotation.ThrowIfNull("未找到对应报告!");
            var report = conn.SingleById<Report>(ReportKvid);
            if(report.AmountSettlement>0) throw new Exception("当前报告已结算，无法修改套餐费！");

            var rtns = new RestfulQueryResponse<ReportQuotation>();
            rtns.Results = new List<ReportQuotation>();

            //if (reportQuotation.Status != 100) throw new Exception("当前报告非委托状态，无法修改套餐费！");
            reportQuotation.QuotationAmount = QuotationAmount;
            reportQuotation.AddOnlyProperties(o => o.QuotationAmount);
            conn.UpdateOnly(reportQuotation);

            rtns.Results.Add(reportQuotation);
            return rtns;
        }
    }

    [Api(Description = "批量更改协议价")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class ReportQuotationMultiUpdateAmounts : RestfulExecution<ReportQuotation>
    {
        public List<ReportQuotation> Items { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            Items.ThrowIfNullOrEmpty("请传入要修改的报告信息!");
            Items.Exists(o => o.Kvid == Guid.Empty || o.QuotationAmount <= 0).ThrowIfTrue("存在错误信息,协议价不能少于0");

            var reportKvids = Items.ConvertAll(o => o.Kvid);
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var query = conn.From<Report>();
            query.Where(o => Sql.In(o.Kvid, reportKvids));
            query.Select(o=>new
            {
                o.Kvid,o.ReportId,o.AmountSettlement,o.QuotationAmount
            });
            var reports = conn.Select(query);
            reports.ThrowIfNullOrEmpty("未找到指定报告信息!");
            if (reports.Count != reportKvids.Count) throw new Exception("传入报告数量与查询到报告数量不一致!");
            if (reports.Exists(o => o.AmountSettlement > 0)) throw new Exception("当前报告已结算，无法修改套餐费！");

            var rtns = new RestfulQueryResponse<ReportQuotation>();
            rtns.Results = new List<ReportQuotation>();

            foreach (var report in reports)
            {
                var item = Items.FirstOrDefault(o => o.Kvid == report.Kvid);
                if (item == null) continue;
                var quo = new ReportQuotation();
                quo.Kvid = report.Kvid;
                quo.QuotationAmount = item.QuotationAmount;
                quo.AddOnlyProperties(o => o.QuotationAmount);
                conn.UpdateOnly(quo);
                rtns.Results.Add(quo);
            }

            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }

    public class ReadExcelResponse<T> : RestfulQueryResponse<T>
    {
        public bool success { get; set; }
    }

    [Api(Description = "读取Excel更改协议价")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class ReportQuotationMultiReadExcel : RestfulExecution<ReportQuotation>
    {
        public override object OnExecution(IRequest req, IResponse res)
        {
            (Request.Files.Count() < 1 || Request.Files == null).ThrowIfTrue("未找到excel文件!");
            var httpFile = Request.Files[0];
            ExcelPackage package = new ExcelPackage(Request.Files[0].InputStream);
            (package.Workbook.Worksheets.Count == 0).ThrowIfTrue("未找到Sheet!");

            var rtns = new ReadExcelResponse<Report>();
            rtns.Results = new List<Report>();
            try
            {
                var sheet = package.Workbook.Worksheets[1];
                var end = sheet.Dimension.End.Row;

                var lists= new List<Report>();
                for (int i = 2; i <= end; i++)
                {
                    #region 格式化Excel中每行数据,必填项若不符合验证条件就抛出异常
                    var eReportId = sheet.Cells[i, 2].Value;
                    if (eReportId == null)
                    {
                        continue;
                        //throw new Exception($"Excel第{i}行第2列不可为空,请传入不重复的报告号!");
                    }
                    var reportId = eReportId.ToString();
                    var eQuotationAmount = sheet.Cells[i, 3].Value;
                    if (eQuotationAmount == null)
                    {
                        throw new Exception($"Excel第{i}行第3列不可为空,请传入协议价格!");
                    }
                    if (!Decimal.TryParse(eQuotationAmount.ToString(), out var amount))
                    {
                        throw new Exception($"Excel第{i}行第3列金额格式不正确!");
                    }
                    if (amount <= 0)
                    {
                        throw new Exception($"Excel第{i}行第3列金额不能小于等于零!");
                    }
                    #endregion
                    var quo = new Report();
                    quo.ReportId = reportId;
                    quo.QuotationAmount = amount;
                    if (lists.Exists(o => o.ReportId == quo.ReportId))
                    {
                        throw new Exception($"报告号{quo.ReportId}在Excel中重复出现!");
                    }
                    lists.Add(quo);
                }

                var reportIds = lists.ConvertAll(o => o.ReportId);
                var conn = KiviiContext.GetOpenedDbConnection<Report>();
                var query = conn.From<Report>();
                query.Where(o => Sql.In(o.ReportId, reportIds));
                query.Select(o => new
                {
                    o.Kvid,
                    o.ReportId,
                    o.QuotationAmount,
                    o.AmountSettlement
                });
                var reports = conn.Select(query);

                foreach (var item in lists)
                {
                    var report = reports.FirstOrDefault(o => o.ReportId == item.ReportId);
                    if (report == null) throw new Exception($"未找到报告{item.ReportId}！");
                    if (report.AmountSettlement > 0) throw new Exception($"报告{report.ReportId}已结算，无法修改协议费！");
                    if (report.QuotationAmount <=0) throw new Exception($"报告{report.ReportId}无协议费,无法修改协议费！");
                    item.Kvid = report.Kvid;
                    item.Amount = report.QuotationAmount;//记录原本的协议费用

                    rtns.Results.Add(item);
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            rtns.Total = rtns.Results.Count;
            rtns.success = true;
            return rtns;
        }
    }
    #endregion

    #region 拷贝粘贴报告
    public enum PasteType
    {
        Unsupported = 0,// 不支持的类型
        ReportItemsOriginal,//粘贴原项目
        ReportItemsAll//粘贴所有项目
    }
    public class MatchKvids
    {
        public Guid OriginalKvid { get; set; }
        public Guid MatchKvid { get; set; }
    }

    [Api(Description = "拷贝粘贴报告(升级)")]
    //复制的报告自动匹配标准库
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class ReportPasteUpgrade : RestfulExecution<Report>
    {
        public Guid FromKvid { get; set; }
        //快递单号
        public bool IsCopySampleSourceBatchNumber { get; set; }
        //样品图片
        public bool IsContainSampleFiles { get; set; }
        //报告要求
        public bool IsCopyReportRequire { get; set; }
        //客户图片
        public bool IsContainReportRequireFiles { get; set; }
        //客户要求
        public bool IsCopyCustomerRequire { get; set; }
        public PasteType Type { get; set; }
        public bool IsContainParts { get; set; }
        public bool IsCopyDeteValueCustomerRequired { get; set; }
        //是否拷贝客户要求,默认不拷贝
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            FromKvid.ThrowIfEmpty("未传入拷贝报告的信息!");
            #endregion
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var connStandards = KiviiContext.GetOpenedDbConnection<Standard>();
            var connProducts = KiviiContext.GetOpenedDbConnection<Quotation>();
            //获取当前拷贝报告
            var report = conn.SingleById<Report>(FromKvid);
            report.ThrowIfNull("未找到拷贝报告!");
            //获取当前拷贝报告对应样品
            var querySamples = conn.From<Sample>();
            querySamples.Where(o => o.RootKvid == report.SampleKvid);
            var samples = conn.Select(querySamples);
            //获取当前拷贝报告对应的检验项目
            var queryReportItems = conn.From<ReportItem>();
            queryReportItems.Where(o => o.ReportKvid == FromKvid);
            var reportItems = conn.Select(queryReportItems);
            //获取当前拷贝报告对应的检验项目中涉及到的标准值信息
            var detectionKvids = reportItems.ConvertAll(o => o.DetectionKvid);
            var queryDetections = connStandards.From<Detection>();
            queryDetections.Where(o => Sql.In(o.Kvid, detectionKvids));
            var detections = connStandards.Select(queryDetections);
            //获取标准信息(涉及到标准信息变更)
            var standardKvids = detections.ConvertAll(o => o.StandardKvid).Distinct().ToList();
            var standards = connStandards.Select<Standard>(o => Sql.In(o.Kvid, standardKvids));
            //获取报告所涉及项目的标准值
            var queryDetectionValues = connStandards.From<DetectionValue>();
            queryDetectionValues.Where(o => Sql.In(o.DetectionKvid, detectionKvids));
            var detectionValues = connStandards.Select(queryDetectionValues);

            //获取报告涉及的合同、服务相关信息
            Quotation quotation = null;
            if (report.QuotationKvid != Guid.Empty) quotation = connProducts.SingleById<Quotation>(report.QuotationKvid);
            List<QuotationDetail> quotationDetails = new List<QuotationDetail>();

            if (quotation != null)
            {
                var queryQuotationDetails = connProducts.From<QuotationDetail>();
                queryQuotationDetails.Where(o => o.QuotationKvid == quotation.Kvid);
                quotationDetails = connProducts.Select(queryQuotationDetails);
            }
            //检验项目后改价格,按最新匹配 2024-03-25
            //goodsKvids:ReportItems的GoodsKvid集合+涉及Detections的GoodsKvid集合(存在标准检验项目后改价格的情况)
            var goodsKvidsByReportItem = reportItems.Select(o => o.GoodsKvid).Distinct().ToList();
            var goodsKvidsByDetections = detections.Select(o => o.GoodsKvid).Distinct().ToList();
            var goodsKvids = goodsKvidsByReportItem.Union(goodsKvidsByDetections).Distinct().ToList();
            var queryPrices = connProducts.From<Price>();
            queryPrices.Where(o => Sql.In(o.GoodsKvid, goodsKvids));
            var prices = connProducts.Select(queryPrices);

            var rtns = new RestfulQueryResponse<Report>();
            rtns.Results = new List<Report>();
            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            using (KiviiContext.Profiler($"粘贴报告：{report.ReportId}"))
            {
                try
                {
                    //1.创建报告+样品
                    var sampleKvid = Guid.NewGuid();
                    //1.1 创建报告
                    var pasteReport = report.GetPasteReport();
                    if (IsCopyCustomerRequire&& report.Metadata.ContainsKey("CustomerRequire")) pasteReport.Metadata["CustomerRequire"] = report.Metadata["CustomerRequire"];
                    //获取原报告Note信息
                    var queryReportNote = conn.From<EntityNote<Report>>();
                    queryReportNote.Where(o => o.InternalCode == "report" && o.EntityKvid == report.Kvid).OrderByDescending(o => o.CreateTime);
                    var reportNote = conn.Select(queryReportNote).FirstOrDefault();
                    if (reportNote != null)
                    {
                        var pasteReportNote = new EntityNote<Report>();
                        pasteReportNote.PopulateWith(reportNote);
                        pasteReportNote.EntityKvid = pasteReport.Kvid;
                        pasteReportNote.Kvid = Guid.NewGuid();
                        conn.Insert(pasteReportNote);
                    }
                    if (IsCopySampleSourceBatchNumber)
                    {
                        pasteReport.SampleSourceKvid = report.SampleSourceKvid;
                        pasteReport.SampleSourceBatchNumber = report.SampleSourceBatchNumber;
                    }
                    if (IsCopyReportRequire)
                    {
                        if (pasteReport.Metadata != null && report.Metadata != null && report.Metadata.ContainsKey("ReportRequire")) pasteReport.Metadata["ReportRequire"] = report.Metadata["ReportRequire"];
                    }
                    if (Type == PasteType.ReportItemsAll)
                    {
                        if (pasteReport.Metadata != null && report.Metadata != null && report.Metadata.ContainsKey("testStandardsMethod")) pasteReport.Metadata["testStandardsMethod"] = report.Metadata["testStandardsMethod"];
                        pasteReport.TestStandards = report.TestStandards;
                        pasteReport.TestDetections = report.TestDetections;
                        pasteReport.AmountTest = report.AmountTest;
                        pasteReport.AmountTestPlan = report.AmountTestPlan;
                    }
                    pasteReport.SampleKvid = sampleKvid;
                    conn.Insert(pasteReport);
                    pasteReport.RemoveAllOnlyProperties();
                    //如果项目名称发生变化,是否考虑重新生成检验依据

                    //1.2 创建主样品
                    var sample = samples.Where(o => o.Kvid == o.RootKvid).FirstOrDefault();
                    sample.ThrowIfNull("未找到拷贝报告主样品!");
                    var pasteSample = new Sample();
                    pasteSample.PopulateWith(sample);
                    pasteSample.Kvid = sampleKvid;
                    pasteSample.RootKvid = sampleKvid;
                    pasteSample.SourceKvid = pasteReport.SampleSourceKvid;
                    pasteSample.SourceBatchNumber = pasteReport.SampleSourceBatchNumber;
                    pasteSample.SerialNumber = pasteReport.ReportId;
                    conn.Insert(pasteSample);
                    //粘贴样品图片(或其他类型文件)
                    if (IsContainSampleFiles)
                    {

                        var querySampleFiles = conn.From<EntityDbFile<Sample>>();
                        querySampleFiles.Where(o => o.DbFolderPath == "/Samples/Images" && o.ParentKvid == Guid.Empty && o.OwnerKvid == report.SampleKvid);
                        var sampleFiles = conn.Select(querySampleFiles);
                        foreach (var sampleFile in sampleFiles)
                        {
                            var pasteSampleFile = new EntityDbFile<Sample>();
                            pasteSampleFile.PopulateWith(sampleFile);
                            pasteSampleFile.OwnerKvid = pasteSample.Kvid;
                            pasteSampleFile.Version = (decimal)0.01;
                            pasteSampleFile.Kvid = Guid.NewGuid();
                            conn.Insert(pasteSampleFile);
                        }
                    }
                    ////粘贴客户图片(或其他类型文件)
                    if (IsContainReportRequireFiles)
                    {
                        var queryReportRequireFiles = conn.From<EntityDbFile<Report>>();
                        queryReportRequireFiles.Where(o => o.DbFolderPath == "/Report/Images" && o.ParentKvid == Guid.Empty && o.OwnerKvid == report.Kvid);
                        var ReportRequireFiles = conn.Select(queryReportRequireFiles);
                        foreach (var file in ReportRequireFiles)
                        {
                            var pasteReportRequireFile = new EntityDbFile<Report>();
                            pasteReportRequireFile.PopulateWith(file);
                            pasteReportRequireFile.OwnerKvid = pasteReport.Kvid;
                            pasteReportRequireFile.Version = (decimal)0.01;
                            pasteReportRequireFile.Kvid = Guid.NewGuid();
                            conn.Insert(pasteReportRequireFile);
                        }
                    }
                    //2.粘贴所有项目
                    if (Type == PasteType.ReportItemsAll)
                    {
                        //2.1 创建样品(可能存在子样品)
                        //获取样品新老Kvid集合
                        var sampleMatchKvids = new List<MatchKvids>();
                        sampleMatchKvids.Add(new MatchKvids
                        {
                            OriginalKvid = Guid.Empty,
                            MatchKvid = Guid.Empty
                        });
                        sampleMatchKvids.Add(new MatchKvids
                        {
                            OriginalKvid = report.SampleKvid,
                            MatchKvid = sampleKvid
                        });
                        //获取当前拷贝报告对应的子样品
                        //获取拷贝检验项目引用的样品集合
                        var sampleKvidsByReportItems = reportItems.Select(o => o.SampleKvid).Distinct().ToList();
                        var sampleChildren = samples.Where(o => Sql.In(o.Kvid, sampleKvidsByReportItems) && o.Kvid != report.SampleKvid).ToList();
                        foreach (var sampleChild in sampleChildren)
                        {
                            var pasteSampleChild = new Sample();
                            pasteSampleChild.PopulateWith(sampleChild);
                            pasteSampleChild.Kvid = Guid.NewGuid();
                            pasteSampleChild.RootKvid = pasteSample.RootKvid;
                            pasteSampleChild.ParentKvid = sampleKvid;
                            pasteSampleChild.SourceKvid = pasteReport.SampleSourceKvid;
                            pasteSampleChild.SourceBatchNumber = String.Empty;
                            pasteSampleChild.SerialNumber = sampleChild.SerialNumber.Replace(sample.SerialNumber, pasteSample.SerialNumber);
                            conn.Insert(pasteSampleChild);
                            sampleMatchKvids.Add(new MatchKvids
                            {
                                OriginalKvid = sampleChild.Kvid,
                                MatchKvid = pasteSampleChild.Kvid
                            });
                        }
                        //2.2 创建检验项目
                        //获取检验项目新老Kvid集合
                        var reportItemsKvids = reportItems.Select(o => o.Kvid).ToList();
                        var reportItemMatchKvids = new List<MatchKvids>();
                        reportItemMatchKvids.Add(new MatchKvids
                        {
                            OriginalKvid = Guid.Empty,
                            MatchKvid = Guid.Empty
                        });
                        foreach (var reportItemKvid in reportItemsKvids)
                        {
                            reportItemMatchKvids.Add(new MatchKvids
                            {
                                OriginalKvid = reportItemKvid,
                                MatchKvid = Guid.NewGuid()
                            });
                        }
                        foreach (var reportItem in reportItems)
                        {
                            var detection = detections.Where(o => o.Kvid == reportItem.DetectionKvid).FirstOrDefault();
                            var pasteReportItem = reportItem.GetPasteReportItemUpgrade(pasteReport,detection, detectionValues,standards,IsCopyDeteValueCustomerRequired,prices);
                            pasteReportItem.RootKvid = reportItem.RootKvid.GetMatchKvid(reportItemMatchKvids);
                            pasteReportItem.ParentKvid = reportItem.ParentKvid.GetMatchKvid(reportItemMatchKvids);
                            pasteReportItem.Kvid = reportItem.Kvid.GetMatchKvid(reportItemMatchKvids);
                            pasteReportItem.ReportKvid = pasteReport.Kvid;
                            pasteReportItem.SampleKvid = reportItem.SampleKvid.GetMatchKvid(sampleMatchKvids);
                            pasteReportItem.SampleRootKvid = reportItem.SampleRootKvid.GetMatchKvid(sampleMatchKvids);
                            pasteReportItem.CalculateAmount(quotation, quotationDetails, prices);
                            conn.Insert(pasteReportItem);
                            //创建标准值日志
                            pasteReportItem.SetDetectionValueQualifiedLog("create", conn,null,null, "ReportPasteUpgrade");
                            //创建客户要求标准值日志
                            if(pasteReportItem.Metadata!=null&&pasteReportItem.Metadata.ContainsKey("DetectionValueCustomer")&& !pasteReportItem.Metadata["DetectionValueCustomer"].IsNullOrEmpty()) pasteReportItem.SetDetectionValueQualifiedCustomerRequiredLog("create",null, conn);
                        }
                    }
                    //3.粘贴原项目
                    if (Type == PasteType.ReportItemsOriginal)
                    {
                        //找到主色号下的所有项
                        var reportItemRoot = reportItems.Where(o => o.SampleKvid == o.SampleRootKvid && o.Type == ReportItemType.SampleSelf).FirstOrDefault();
                        //获取需要拷贝的所有项目
                        var reportItemsOriginal = new List<ReportItem>();
                        var isCopyAllReportItems = false;
                        if (reportItemRoot == null)
                        {
                            reportItemsOriginal = reportItems;
                            isCopyAllReportItems = true;
                        }
                        else
                        {
                            //获取主色号下的所有项目
                            reportItemsOriginal = reportItems.Where(o => o.RootKvid == reportItemRoot.Kvid && o.ParentKvid != Guid.Empty).ToList();
                        }
                        //获取检验项目新老Kvid集合
                        var reportItemOriginalKvids = reportItemsOriginal.Select(o => o.Kvid).ToList();
                        var reportItemOriginalMatchKvids = new List<MatchKvids>();
                        reportItemOriginalMatchKvids.Add(new MatchKvids
                        {
                            OriginalKvid = Guid.Empty,
                            MatchKvid = Guid.Empty
                        });
                        foreach (var reportItemKvid in reportItemOriginalKvids)
                        {
                            reportItemOriginalMatchKvids.Add(new MatchKvids
                            {
                                OriginalKvid = reportItemKvid,
                                MatchKvid = Guid.NewGuid()
                            });
                        }
                        if (IsContainParts)
                        {
                            //包含部位
                            //创建检验项目
                            foreach (var reportItemOriginal in reportItemsOriginal)
                            {
                                var detection = detections.Where(o => o.Kvid == reportItemOriginal.DetectionKvid).FirstOrDefault();
                                var pasteReportItem = reportItemOriginal.GetPasteReportItemUpgrade(pasteReport, detection, detectionValues, standards, IsCopyDeteValueCustomerRequired,prices);
                                //判断主项/子项
                                var newRootKvid = new Guid();
                                var newParentKvid = new Guid();
                                if (isCopyAllReportItems)
                                {
                                    newRootKvid = reportItemOriginal.RootKvid;
                                    newParentKvid = reportItemOriginal.ParentKvid;
                                }
                                else
                                {
                                    if (reportItemOriginal.ParentKvid == reportItemOriginal.RootKvid)
                                    {
                                        //主项
                                        newRootKvid = reportItemOriginal.Kvid;
                                        newParentKvid = Guid.Empty;
                                    }
                                    else
                                    {
                                        //子项,子项的RootKvid需要特殊处理
                                        newRootKvid = reportItemOriginal.GetRootKvid(reportItemsOriginal);
                                        newParentKvid = reportItemOriginal.ParentKvid;
                                    }
                                }
                                pasteReportItem.RootKvid = newRootKvid.GetMatchKvid(reportItemOriginalMatchKvids);
                                pasteReportItem.ParentKvid = newParentKvid.GetMatchKvid(reportItemOriginalMatchKvids);
                                pasteReportItem.Kvid = reportItemOriginal.Kvid.GetMatchKvid(reportItemOriginalMatchKvids);
                                pasteReportItem.ReportKvid = pasteReport.Kvid;
                                pasteReportItem.SampleKvid = sampleKvid;
                                pasteReportItem.SampleRootKvid = sampleKvid;
                                pasteReportItem.CalculateAmount(quotation, quotationDetails, prices);
                                conn.Insert(pasteReportItem);
                                //创建标准值日志
                                pasteReportItem.SetDetectionValueQualifiedLog("create", conn,null,null, "ReportPasteUpgrade");
                                //创建客户要求标准值日志
                                if (pasteReportItem.Metadata != null && pasteReportItem.Metadata.ContainsKey("DetectionValueCustomer") && !pasteReportItem.Metadata["DetectionValueCustomer"].IsNullOrEmpty()) pasteReportItem.SetDetectionValueQualifiedCustomerRequiredLog("create",null, conn);
                            }

                        }
                        else
                        {
                            //不包含部位,提取所有DetectionKvid不为空的数据
                            //只保留主项(去重)->处理对应子项->处理剩下的Type为Test的主项
                            reportItemsOriginal = reportItemsOriginal.Where(o => o.DetectionKvid != Guid.Empty).ToList();
                            var reportItemsOriginalTestFolder = reportItemsOriginal.Where(o => o.Type == ReportItemType.TestFolder).ToList();
                            reportItemsOriginal = reportItemsOriginal.Except(reportItemsOriginalTestFolder).ToList();
                            //记录处理过的主项值
                            var detectionKvidsExist = new List<Guid>();
                            foreach (var reportItemOriginalTestFolder in reportItemsOriginalTestFolder)
                            {
                                //如遇相同主项直接跳过
                                if (detectionKvidsExist.Contains(reportItemOriginalTestFolder.DetectionKvid))
                                {
                                    reportItemsOriginal.Remove(reportItemOriginalTestFolder);
                                    var reportItemsTestFolderChildren = reportItemsOriginal.Where(o => o.ParentKvid == reportItemOriginalTestFolder.Kvid).ToList();
                                    reportItemsOriginal = reportItemsOriginal.Except(reportItemsTestFolderChildren).ToList();
                                    continue;
                                }
                                detectionKvidsExist.Add(reportItemOriginalTestFolder.DetectionKvid);
                                //创建主项
                                var detection = detections.Where(o => o.Kvid == reportItemOriginalTestFolder.DetectionKvid).FirstOrDefault();
                                var pasteReportItemParent = reportItemOriginalTestFolder.GetPasteReportItemUpgrade(pasteReport, detection, detectionValues, standards, IsCopyDeteValueCustomerRequired,prices);
                                pasteReportItemParent.RootKvid = reportItemOriginalTestFolder.Kvid.GetMatchKvid(reportItemOriginalMatchKvids);
                                pasteReportItemParent.ParentKvid = Guid.Empty;
                                pasteReportItemParent.Kvid = reportItemOriginalTestFolder.Kvid.GetMatchKvid(reportItemOriginalMatchKvids);
                                pasteReportItemParent.ReportKvid = pasteReport.Kvid;
                                pasteReportItemParent.SampleKvid = sampleKvid;
                                pasteReportItemParent.SampleRootKvid = sampleKvid;
                                pasteReportItemParent.CalculateAmount(quotation, quotationDetails, prices);
                                conn.Insert(pasteReportItemParent);
                                reportItemsOriginal.Remove(reportItemOriginalTestFolder);
                                //创建标准值日志
                                pasteReportItemParent.SetDetectionValueQualifiedLog("create", conn,null,null, "ReportPasteUpgrade");
                                //创建客户要求标准值日志
                                if (pasteReportItemParent.Metadata != null && pasteReportItemParent.Metadata.ContainsKey("DetectionValueCustomer") && !pasteReportItemParent.Metadata["DetectionValueCustomer"].IsNullOrEmpty()) pasteReportItemParent.SetDetectionValueQualifiedCustomerRequiredLog("create",null, conn);
                                //创建子项
                                var reportItemsOriginalChildren = reportItemsOriginal.Where(o => o.ParentKvid == reportItemOriginalTestFolder.Kvid).ToList();
                                foreach (var reportItemChild in reportItemsOriginalChildren)
                                {
                                    var detectionChild = detections.Where(o => o.Kvid == reportItemChild.DetectionKvid).FirstOrDefault();
                                    var pasteReportItemChild = reportItemChild.GetPasteReportItemUpgrade(pasteReport, detectionChild, detectionValues, standards, IsCopyDeteValueCustomerRequired,prices);
                                    pasteReportItemChild.RootKvid = pasteReportItemParent.Kvid;
                                    pasteReportItemChild.ParentKvid = pasteReportItemParent.Kvid;
                                    pasteReportItemChild.Kvid = reportItemChild.Kvid.GetMatchKvid(reportItemOriginalMatchKvids);
                                    pasteReportItemChild.ReportKvid = pasteReport.Kvid;
                                    pasteReportItemChild.SampleKvid = sampleKvid;
                                    pasteReportItemChild.SampleRootKvid = sampleKvid;
                                    pasteReportItemChild.CalculateAmount(quotation, quotationDetails, prices);
                                    conn.Insert(pasteReportItemChild);
                                    reportItemsOriginal.Remove(reportItemChild);
                                    //创建标准值日志
                                    pasteReportItemChild.SetDetectionValueQualifiedLog("create", conn, null, null, "ReportPasteUpgrade");
                                    //创建客户要求标准值日志
                                    if (pasteReportItemChild.Metadata != null && pasteReportItemChild.Metadata.ContainsKey("DetectionValueCustomer") && !pasteReportItemChild.Metadata["DetectionValueCustomer"].IsNullOrEmpty()) pasteReportItemChild.SetDetectionValueQualifiedCustomerRequiredLog("create",null, conn);
                                }

                            }
                            //处理剩余项目:Type为Test的主项
                            foreach (var reportItem in reportItemsOriginal)
                            {
                                //如遇相同主项直接跳过
                                if (detectionKvidsExist.Contains(reportItem.DetectionKvid)) continue;
                                detectionKvidsExist.Add(reportItem.DetectionKvid);
                                var detection = detections.Where(o => o.Kvid == reportItem.DetectionKvid).FirstOrDefault();
                                var pasteReportItem = reportItem.GetPasteReportItemUpgrade(pasteReport,detection, detectionValues, standards, IsCopyDeteValueCustomerRequired, prices);
                                pasteReportItem.RootKvid = reportItem.Kvid.GetMatchKvid(reportItemOriginalMatchKvids);
                                pasteReportItem.ParentKvid = Guid.Empty;
                                pasteReportItem.Kvid = reportItem.Kvid.GetMatchKvid(reportItemOriginalMatchKvids);
                                pasteReportItem.ReportKvid = pasteReport.Kvid;
                                pasteReportItem.SampleKvid = sampleKvid;
                                pasteReportItem.SampleRootKvid = sampleKvid;
                                pasteReportItem.CalculateAmount(quotation, quotationDetails, prices);
                                conn.Insert(pasteReportItem);
                                //创建标准值日志
                                pasteReportItem.SetDetectionValueQualifiedLog("create", conn, null, null, "ReportPasteUpgrade");
                                //创建客户要求标准值日志
                                if (pasteReportItem.Metadata != null && pasteReportItem.Metadata.ContainsKey("DetectionValueCustomer") && !pasteReportItem.Metadata["DetectionValueCustomer"].IsNullOrEmpty()) pasteReportItem.SetDetectionValueQualifiedCustomerRequiredLog("create",null, conn);
                            }
                        }
                    }
                    rtns.Total = rtns.Results.Count;
                    trans?.Commit();
                    //事务提交后，同步报告金额
                    pasteReport.CalculateReportAmount(conn);
                    conn.UpdateOnly(pasteReport);
                    pasteReport.RemoveAllOnlyProperties();
                    rtns.Results.Add(pasteReport);
                    return rtns;
                }
                catch (Exception ex)
                {
                    trans?.Rollback();
                    throw ex;
                }
            }
        }
    }
    #endregion

    #region 报告下达验证，接口支持
    public class ReportReleaseValidResponse : RestfulResponse
    {
        public bool IsSucceed { get; set; }
        public string Messages { get; set; }
    }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class ReportReleaseValid : RestfulExecution<Report>
    {
        public List<Guid> ReportKvids { get; set; }
        public ReportStatusType StatusType { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportKvids.ThrowIfNullOrEmpty("未传入报告信息！");
            StatusType.ThrowIfNull("未传入下达状态！");
            //批量下达报告数量不能超过10
            var count = ReportKvids.Count;
            (count > 10).ThrowIfTrue("批量下达报告数量不能超过“10”！");

            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var queryReports = conn.From<Report>();
            queryReports.Where(o => Sql.In(o.Kvid, ReportKvids));
            var reports = conn.Select(queryReports);
            reports.ThrowIfNullOrEmpty("未查询到任何报告！");

            var rtns = new ReportReleaseValidResponse();
            List<string> exList = new List<string>();
            //处理委托受理的验证
            if (StatusType == ReportStatusType.CommissionAccept)
            {
                //验证项目中“项目关联已变更,请删除重新选择”信息，禁止下达
                if (conn.Exists<ReportItem>(o => Sql.In(o.ReportKvid, ReportKvids) && o.PackageUnit == "价格关联已变更,请删除重新选择")) throw new Exception("下达失败!<br/>拷贝项目存在价格变更的情况,请删除后重新选择!");
                //如果传入的报告中含有不符合当前下达状态的报告，抛异常
                var difReports = reports.Where(o => o.StatusType != StatusType).Select(o => o.ReportId).ToList();
                if (difReports.Count != 0)
                {
                    var str = string.Join(",", difReports);
                    throw new Exception("报告" + str + "与当前状态不符，无法下达！");
                }
                foreach (var report in reports)
                {
                    //项目为空不能下达
                    if (report.TestDetections.IsNullOrEmpty()) throw new Exception($"报告：{report.ReportId}检验项目为空，不能下达！");
                    //验证报告模板是否过期
                    report.TemplateKvid.ThrowIfEmpty($"报告：{report.ReportId}报告模板选择异常，请重新选择报告模板！如重复遇到，请联系系统管理员！");
                    var template = conn.SingleById<Template>(report.TemplateKvid);
                    if (template != null && DateTime.Now > template.ExpiredTime&&report.TestStartDate == null) throw new Exception($"报告：{report.ReportId}模板已失效，请手动重新选择模板！");
                    //当填写了纤维成分时，验证数字相加等于100
                    if (report.Metadata != null && report.Metadata.ContainsKey("FibreContent") && report.Metadata["FibreContent"] != string.Empty)
                    {
                        var fibrStr = report.Metadata["FibreContent"].Replace("面料1", "").Replace("面料2", "").Replace("里料1", "").Replace("里料2", "").Replace("纱线1", "").Replace("纱线2", "").Replace("配料1", "").Replace("配料2", "").Replace("填充物1", "").Replace("填充物2", "");
                        string result = fibrStr.IsNumber();
                        string[] fRtns = result.Split(new char[1] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
                        decimal decl = 0;

                        foreach (var rtn in fRtns)
                        {
                            if (string.IsNullOrEmpty(rtn)) continue;
                            decl += decimal.Parse(rtn);
                        }
                        (decl != 100&& decl%100!=0).ThrowIfTrue($"纤维含量的总和[{decl}!=100]数据不正确,请认真核对录入的数据信息！");
                    }
                        
                    ////当送样方式为邮寄的时候，需要维护快递单号
                    //if (report.Metadata != null && report.Metadata.ContainsKey("SampleMode") && report.Metadata["SampleMode"] == "邮寄")
                    //{
                    //    if(report.SampleSourceBatchNumber.IsNullOrEmpty()) exList.Add($"报告：{report.ReportId}到样方式为邮寄，需要维护快递单号");
                    //}
                    ////报告号为-Y要选中英文模板
                    //if (report.ReportId.Substring(report.ReportId.Length - 2) == "-Y"&&!report.TemplateName.Contains("英文"))
                    //{
                    //    exList.Add($"报告：{report.ReportId}是英文报告，但未选择英文报告的模板！");
                    //}
                    //若检验项目里包含抑菌率等项目，需要使用微生物模板
                    if((report.TestDetections.Contains("抑菌率")|| report.TestDetections.Contains("防螨性能") || report.TestDetections.Contains("微生物")) && !report.TemplateName.Contains("微生物") && !report.TemplateName.Contains("CNAS"))
                    {
                        exList.Add($"报告：{report.ReportId}包含微生物项目，但未选择微生物报告的模板！");
                    }
                    //委托日期不能为空
                    if (report.DeadDate == null) exList.Add($"报告：{report.ReportId}未选择委托日期");
                    if (report.DeadDate == null) exList.Add($"报告：{report.ReportId}未选择完成日期");
                    //判别所选合同是否有效
                    if (report.QuotationKvid != Guid.Empty)
                    {
                        //处理-G报告跳过不验证合同有效性
                        if (report.ReportId.Count(o => o == '-') <2)
                        {
                            var connQuotation = KiviiContext.GetOpenedDbConnection<Quotation>();
                            var quotation = conn.SingleById<Quotation>(report.QuotationKvid);
                            if (quotation == null) exList.Add($"报告：{report.ReportId}合同不存在！请重新选择合同！");
                            if (quotation.Status != 200) exList.Add($"报告：{report.ReportId}合同无效，请重新选择合同！");
                        }
                        
                    }
                    ////★项目下达时，必须要在备注中标记“星号项目未经实验室认可”
                    //if (report.TestDetections.Contains("★")||conn.Count<ReportItem>(o=>o.ReportKvid == report.Kvid&&o.Title.Contains("★"))>0)
                    //{
                    //    (report.Remark.IsNullOrEmpty() || (!report.Remark.Contains("星号项目未经实验室认可")&& !report.Remark.Contains("星号项目未经CNAS认可"))).ThrowIfTrue($"报告：{report.ReportId}存在星号项目，需备注【星号项目未经CNAS认可】！");
                    //}
                    ////验证检验项目中的报告要求，如果有值，则必须客户要求要有值
                    //var currentItems = report.GetOriginalReportItems();
                    //foreach(var reportItem in currentItems)
                    //{
                    //    if (reportItem.Metadata != null && reportItem.Metadata.ContainsKey("DetectionValueCustomer") && reportItem.Metadata["DetectionValueCustomer"] != null&& reportItem.Metadata["DetectionValueCustomer"] != "")
                    //    {
                    //        if(!report.Metadata.ContainsKey("CustomerRequire")|| report.Metadata["CustomerRequire"] == null || report.Metadata["CustomerRequire"] == "")
                    //        {
                    //            exList.Add($"报告：{report.ReportId}的{reportItem.Title}存在客户要求，但报告基本信息中无客户要求，请检查！");
                    //        }
                    //    }
                    //}
                }

            }
            //if(StatusType == ReportStatusType.DataEntry)
            //{
            //    //根据角色获取当前人所属的角色列表
            //    //获取duty表中对应的角色
            //    var queryTestRoles = conn.From<EntityDuty<Test>>();
            //    queryTestRoles.Where(o => o.OwnerKvid == KiviiContext.CurrentMember.Kvid);
            //    queryTestRoles.Select(o => o.RoleName);
            //    var testRoleNames = conn.Select(queryTestRoles).ConvertAll(o => o.RoleName);
            //    foreach (var report in reports)
            //    {
            //        //监督报告包含不合格项目时，要提示
            //        if (report.TypeEx == "J")
            //        {
            //            var countUnqualified = conn.Count<ReportItem>(o => o.ReportKvid == report.Kvid && o.Type == ReportItemType.Test && Sql.In(o.WorkGroupName, testRoleNames) && o.Judgement == "不合格");
            //            if (countUnqualified > 0) exList.Add($"报告：{report.ReportId}为监督报告，存在不合格项目，请确认经过复测！");
            //        }
            //        //只处理纤维含量组的验证
            //        if (testRoleNames.Exists(o => o == "纤维含量组"))
            //        {
            //            var reportItems = report.GetOriginalReportItems();
            //            var fibrTests = reportItems.Where(o => (o.Title.Contains("纤维含量") || o.Title.Contains("Fibre Content")) && o.WorkGroupName == "纤维含量组" && !o.TestValue.IsNullOrEmpty() && o.TestValue != "/").ToList();
            //            foreach (var fibre in fibrTests)
            //            {
            //                if (!fibre.TestValue.Contains("100") && !fibre.Title.Contains("公定"))
            //                {
            //                    exList.Add($"{report.ReportId}项目名称不存在“公定”，请确认检测结果与项目名称是否匹配！");
            //                }
            //            }
            //        }
            //    }
            //}
            if (exList.Count != 0)
            {
                rtns.IsSucceed = false;
                rtns.Messages = string.Join(";", exList);
            }
            else
            {
                rtns.IsSucceed = true;
            }
            return rtns;
        }
    }
    #endregion

    #region 静默更新报告检验项目的检验依据
    //只处理有强标的报告18401，31701,20400,30585
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class CaculateReportItemStandardCode : RestfulExecution<Report>
    {
        public List<Report> reports { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            if (reports.IsNullOrEmpty()) return null;
            var connLims = KiviiContext.GetOpenedDbConnection<Report>();
            var connStandard = KiviiContext.GetOpenedDbConnection<Standard>();
            var reportKvids = reports.Select(o => o.Kvid).ToList();

            var queryAllReports = connLims.From<Report>();
            queryAllReports.Where(o => Sql.In(o.Kvid, reportKvids));
            var allReports = connLims.Select(queryAllReports);

            var queryAllReportItems = connLims.From<ReportItem>();
            queryAllReportItems.Where(o => Sql.In(o.ReportKvid, reportKvids));
            var allReportItems = connLims.Select(queryAllReportItems);

            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            foreach (var report in allReports)
            {
                if (report.Kvid == Guid.Empty) continue;
                //报告的检验依据无强标，直接跳过不处理
                //找出报告的所有强标
                var reportTestStandards = report.TestStandards;
                List<string> forceStandards = new List<string>();
                if (reportTestStandards.Contains("18401")) forceStandards.Add("GB 18401-2010");
                if (reportTestStandards.Contains("31701")) forceStandards.Add("GB 31701-2015");
                if (reportTestStandards.Contains("20400")) forceStandards.Add("GB 20400-2006");
                if (reportTestStandards.Contains("30585")) forceStandards.Add("GB 30585-2014");
                if (forceStandards.IsNullOrEmpty()) continue;

                var currentReportItems = allReportItems.Where(o => o.ReportKvid == report.Kvid).ToList();
                var currentMainReportItems = currentReportItems.filterReportItem();
                List<ReportItem> updateReportItems = new List<ReportItem>();
                foreach (var standardStr in forceStandards)
                {
                    //查找出该强标下的所有大项
                    var queryStandardKvids = connStandard.From<Standard>();
                    queryStandardKvids.Where(o => o.Code.Contains(standardStr));
                    queryStandardKvids.Select(o => o.Kvid);
                    var queryDetections = connStandard.From<Detection>();
                    queryDetections.Select(o => o.Title);
                    queryDetections.Where(o => Sql.In(o.StandardKvid, queryStandardKvids) && o.ReferenceDetectionKvid != Guid.Empty);
                    var detections = connStandard.Select(queryDetections);
                    //List<string> detectionStr = new List<string>();
                    //foreach(var en in detections)
                    //{
                    //    detectionStr.Add(en.Title);
                    //}
                    foreach (var reportItem in currentMainReportItems)
                    {
                        if (detections.Exists(o => o.Title.Contains(reportItem.Title)))
                        {
                            if (reportItem.Metadata == null) reportItem.Metadata = new Dictionary<string, string>();
                            //非本强标的项目才需要匹配强标
                            if (!reportItem.StandardCode.Contains(standardStr))
                            {
                                if (reportItem.Metadata.ContainsKey("StandardCodeEx"))
                                {
                                    if (!reportItem.Metadata["StandardCodeEx"].Contains(standardStr))
                                    {
                                        reportItem.Metadata["StandardCodeEx"] = reportItem.Metadata["StandardCodeEx"] + "," + standardStr;
                                    }
                                }
                                else reportItem.Metadata.Add("StandardCodeEx", standardStr);
                                reportItem.AddOnlyProperties(o => o.Metadata);
                                updateReportItems.Add(reportItem);
                            }

                        }
                    }
                }
                IDbTransaction trans = null;
                trans = connLims.OpenTransaction();
                try
                {
                    foreach (var item in updateReportItems)
                    {
                        connLims.UpdateOnly(item);
                        rtns.Results.Add(item);
                    }
                    trans?.Commit();
                }
                catch { }
            }
            rtns.Total = rtns.Results.Count();
            return rtns;
        }
    }
    #endregion

    #region 同步报告信息(金额+检验依据等信息)
    //代码初审 张梦圆 20210906
    [Api(Description = "同步报告")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class ReportSyncByStandard : RestfulExecution<Report>
    {
        public Guid ReportKvid { get; set; }
        public bool CombineProductAndMethod { get; set; }
        public bool CalcForceStandard { get; set; }
        public List<string> ForceStandards { get; set; }
        public bool CalcProductStandard { get; set; }
        public bool CalcProductStandardWithoutForce { get; set; }
        public bool CalcMethodStandard { get; set; }
        /// <summary>
        /// 按需计算
        /// </summary>
        public bool NeedCalculateReportDetection { get; set; } = true;
        public bool NeedCalculateReportAmount { get; set; } = true;
        public bool NeedCalculateStandard { get; set; } = true;

        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            ReportKvid.ThrowIfEmpty("请传入要同步的报告!");
            #endregion
            //获取报告信息
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var report = conn.SingleById<Report>(ReportKvid);
            if (report == null) throw new Exception();
            var templateName = report.TemplateName;
            bool hasEnTitle = false;
            if (!templateName.IsNullOrEmpty())
            {
                if (templateName.Contains("UR") || templateName.Contains("中英文")) hasEnTitle = true;
            }
            
            if (NeedCalculateStandard)
            {
                report.CalculateStandard(CombineProductAndMethod, CalcForceStandard, ForceStandards, CalcProductStandard, CalcProductStandardWithoutForce, CalcMethodStandard, hasEnTitle,conn);
            }
            if(NeedCalculateReportDetection) report.CalculateReportDetection(conn);
            if (NeedCalculateReportAmount) report.CalculateReportAmount(conn);
            conn.UpdateOnly(report);
            report.RemoveAllOnlyProperties();
            var rtns = new RestfulQueryResponse<Report>();
            rtns.Results = new List<Report>();
            rtns.Results.Add(report);
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion
}
