﻿ using Kivii.Domain.Lac.Lims.Extensions;
using Kivii;
using Kivii.Linq;
using Kivii.Products.Entities;
using Kivii.Standards.Entities;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Kivii.Organizations.Entities;

namespace Kivii.Domain.Lac.Lims.Transforms
{
    #region 查看检验项目
    //根据StandardKvid或detectionKvids返回带有价格、标准代码、标准名称的detections
    //代码复审 张梦圆 20220144
    [Api(Description = "查看检验项目(添加扩展字段)")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class DetectionWithPriceQuery : RestfulExecution<DetectionEx>
    {
        public Guid StandardKvid { get; set; }
        public List<Guid> DetectionKvids { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            if (StandardKvid == Guid.Empty && DetectionKvids.IsNullOrEmpty()) throw new Exception("参数为空！");
            if (StandardKvid != Guid.Empty && DetectionKvids != null) throw new Exception("不可同时传入StandardKvid和DetectionKvids参数！");
            #endregion

            var connStandards = KiviiContext.GetOpenedDbConnection<Standard>();
            var connGoods = KiviiContext.GetOpenedDbConnection<Product>();

            var rtns = new RestfulQueryResponse<DetectionEx>();
            rtns.Results = new List<DetectionEx>();

            //获取标准&检验项目&检验组&价格集合
            var standards = new List<Standard>();
            var detections = new List<Detection>();
            var taskInfos = new List<EntityRelation<Detection>>();
            var priceLists = new List<Price>();

            //获取检验项目信息
            var queryDetections = connStandards.From<Detection>();
            if (StandardKvid != Guid.Empty)
            {
                queryDetections.Where(o => o.StandardKvid == StandardKvid);
            }
            else
            {
                queryDetections.Where(o => Sql.In(o.Kvid, DetectionKvids) || Sql.In(o.ParentKvid, DetectionKvids));
            }
            detections = connStandards.Select(queryDetections);

            //获取涉及标准信息
            var standardKvids = detections.ConvertAll(o=>o.StandardKvid).Distinct().ToList();
            var queryStandards = connStandards.From<Standard>();
            queryStandards.Where(o=>Sql.In(o.Kvid, standardKvids));
            standards = connStandards.Select(queryStandards);

            //获取检验组信息
            var detectionKvids = detections.ConvertAll(o=>o.Kvid).Distinct().ToList();
            var queryTaskInfos = connStandards.From<EntityRelation<Detection>>();
            queryTaskInfos.Where(o => Sql.In(o.EntityKvid, detectionKvids) && o.RelationType == "Kivii.Organizations.Entities.Organization");
            taskInfos = connStandards.Select(queryTaskInfos);

            //获取价格信息
            var queryPrice = connGoods.From<Price>();
            queryPrice.Where(o => o.IsDefault == true);
            priceLists = connGoods.Select(queryPrice);

            foreach (var detection in detections)
            {
                var detectionEx = new DetectionEx();
                detectionEx.PopulateWith(detection);
                //先处理子项
                if (detection.ParentKvid != Guid.Empty)
                {
                    rtns.Results.Add(detectionEx);
                    continue;
                }
                //获取标准信息
                var standard = standards.FirstOrDefault(o=>o.Kvid== detection.StandardKvid);
                standard.ThrowIfNull("未找到对应标准!");
                detectionEx.StandardCode = standard.Code;
                detectionEx.StandardTitle = standard.Title;
                //获取检验组信息
                //var taskInfo = taskInfos.FirstOrDefault(o => o.EntityKvid == detection.Kvid);
                //if (taskInfo == null)
                //{
                //    detectionEx.TaskGroupFullName = "未设置检验组";
                //    detectionEx.TaskGroupKvid = Guid.Empty;
                //}
                //else
                //{
                //    detectionEx.TaskGroupFullName = taskInfo.InternalCode;
                //    detectionEx.TaskGroupKvid = taskInfo.RelationKvid;
                //}
                //处理主项(已关联服务+未关联服务)
                if (detection.GoodsKvid == Guid.Empty)
                {
                    detectionEx.Amount = -2;
                }
                else
                {
                    var amount = DetectionExtension.GetPrice(detection.GoodsKvid, priceLists);
                    detectionEx.Amount = amount;
                    if (amount == -1)
                    {
                        detectionEx.Metadata["Description"] = "关联的服务未设置价格";
                    }
                }
                rtns.Results.Add(detectionEx);
            }
            rtns.Total = rtns.Results.Count();
            return rtns;
        }
    }
    #endregion

    //应用场景:纺检项目分组
    [Api(Description = "多个项目取消分组")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class DetectionCancelGroup : RestfulExecution<Detection>
    {
        public List<Guid> DetectionKvids { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            var connStds = KiviiContext.GetOpenedDbConnection<Detection>();
            var connOrgs = KiviiContext.GetOpenedDbConnection<Organization>();
            //变更检验项目&组织机构Refs关联数据的状态：-1
            var updateLamdaDetesRefes = connStds.From<EntityRelation<Detection>>();
            updateLamdaDetesRefes = updateLamdaDetesRefes.Update(o => new { o.Status });
            updateLamdaDetesRefes = updateLamdaDetesRefes.Where(o => Sql.In(o.EntityKvid, DetectionKvids));
            connStds.UpdateOnly(new EntityRelation<Detection> { Status = -1 }, updateLamdaDetesRefes);

            var updateLamdaOrgsRefes = connOrgs.From<EntityRelation<Organization>>();
            updateLamdaOrgsRefes = updateLamdaOrgsRefes.Update(o => new { o.Status });
            updateLamdaOrgsRefes = updateLamdaOrgsRefes.Where(o => Sql.In(o.RelationKvid, DetectionKvids));
            connOrgs.UpdateOnly(new EntityRelation<Organization> { Status = -1 }, updateLamdaOrgsRefes);
            //清空检验项目TestGroupKvid和TestGroupFullName信息
            var updateLamdaDetections = connStds.From<Detection>();
            updateLamdaDetections = updateLamdaDetections.Update(o => new { o.TestGroupKvid, o.TestGroupFullName });
            updateLamdaDetections = updateLamdaDetections.Where(o => Sql.In(o.Kvid, DetectionKvids));
            connStds.UpdateOnly(new Detection { TestGroupKvid = Guid.Empty, TestGroupFullName = string.Empty }, updateLamdaDetections);
            return true;
        }
    }
}
